@@ -1,10 +1,32 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
- * GNOME time/date display module.
- * (C) 1997 The Free Software Foundation
+ * clock.c: the GNOME clock applet
  *
- * Authors: George Lebl
- * 	    Gediminas Paulauskas
+ * Copyright (C) 1997-2003 Free Software Foundation, Inc.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ *
+ * Authors:
+ *      Miguel de Icaza
+ *      Frederico Mena
+ *      Stuart Parmenter
+ *      Alexander Larsson
+ *      George Lebl
+ *      Gediminas Paulauskas
+ *      Mark McLoughlin
  */
 
 #include "config.h"
@@ -33,6 +55,8 @@
 
 #define N_GCONF_PREFS 7
 
+#define NEVER_SENSITIVE "never_sensitive"
+
 static const char* KEY_HOUR_FORMAT   = "hour_format";
 static const char* KEY_SHOW_SECONDS  = "show_seconds";
 static const char* KEY_SHOW_DATE     = "show_date";
@@ -49,6 +73,7 @@ struct _ClockData {
 	GtkWidget *clockw;
         GtkWidget *toggle;
 	GtkWidget *props;
+	GtkWidget *about;
   
 	/* preferences */
 	int hourformat;
@@ -379,6 +404,9 @@ destroy_clock(GtkWidget * widget, ClockData *cd)
 		cd->timeout = 0;
 	}
 
+	if (cd->about)
+		gtk_widget_destroy (cd->about);
+
 	if (cd->props) {
 		gtk_widget_destroy (cd->props);
 		cd->props = NULL;
@@ -750,7 +778,7 @@ try_config_tool (GdkScreen  *screen,
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
+					 GTK_BUTTONS_OK,
 					 _("Failed to launch time configuration tool: %s"),
 					 err->message);
 		
@@ -787,7 +815,7 @@ config_date (BonoboUIComponent *uic,
 	dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_ERROR,
-					 GTK_BUTTONS_CLOSE,
+					 GTK_BUTTONS_OK,
 					 _("Failed to locate a program for configuring "
 					   "the date and time. Perhaps none is installed?"));
 		
@@ -1093,11 +1121,37 @@ fill_clock_applet (PanelApplet *applet)
 	return TRUE;
 }
 
+static void
+setup_writability_sensitivity (ClockData *clock, GtkWidget *w, GtkWidget *label, const char *key)
+{
+	char *fullkey;
+	GConfClient *client;
+
+	client = gconf_client_get_default ();
+
+	fullkey = panel_applet_gconf_get_full_key
+		(PANEL_APPLET (clock->applet), key);
+
+	if ( ! gconf_client_key_is_writable (client, fullkey, NULL)) {
+		g_object_set_data (G_OBJECT (w), NEVER_SENSITIVE,
+				   GINT_TO_POINTER (1));
+		gtk_widget_set_sensitive (w, FALSE);
+		if (label != NULL) {
+			g_object_set_data (G_OBJECT (label), NEVER_SENSITIVE,
+					   GINT_TO_POINTER (1));
+			gtk_widget_set_sensitive (label, FALSE);
+		}
+	}
+
+	g_free (fullkey);
+}
+
 static void
 set_data_sensitive_cb (GtkWidget *w,
 		      GtkWidget *wid)
 {
-	gtk_widget_set_sensitive (wid, TRUE);
+	if ( ! g_object_get_data (G_OBJECT (wid), NEVER_SENSITIVE))
+		gtk_widget_set_sensitive (wid, TRUE);
 }
 
 static void
@@ -1203,7 +1257,7 @@ properties_response_cb (GtkWidget *widget,
 			dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
 							 GTK_DIALOG_DESTROY_WITH_PARENT,
 							 GTK_MESSAGE_ERROR,
-							 GTK_BUTTONS_CLOSE,
+							 GTK_BUTTONS_OK,
 							  _("There was an error displaying help: %s"),
 							 error->message);
 
@@ -1240,7 +1294,6 @@ display_properties_dialog (BonoboUIComponent *uic,
 	GtkWidget *menu;
 	GtkWidget *label;
 	GSList    *list;
-	gchar     *title;
 	char      *file;
 
 	if (cd->props) {
@@ -1262,6 +1315,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gtk_window_set_resizable (GTK_WINDOW (cd->props), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (cd->props),
 			       gtk_widget_get_screen (cd->applet));
+	gtk_container_set_border_width (GTK_CONTAINER (cd->props), 5);
 		
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP,
 	                                  "gnome-clock.png", TRUE, NULL);
@@ -1276,7 +1330,7 @@ display_properties_dialog (BonoboUIComponent *uic,
 		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
 
 	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 12);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (cd->props)->vbox), vbox, FALSE, FALSE, 0);
 	gtk_widget_show (vbox);
 
@@ -1284,14 +1338,12 @@ display_properties_dialog (BonoboUIComponent *uic,
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
 	gtk_widget_show (hbox);
 
-	title = g_strconcat ("<span weight=\"bold\">", _("Clock _type:"), "</span>", NULL);
-	label = gtk_label_new_with_mnemonic (title);
+	label = gtk_label_new_with_mnemonic (_("Clock _type:"));
 	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_widget_show (label);
-	g_free (title);
 
 	option_menu = gtk_option_menu_new ();
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), option_menu);
@@ -1429,6 +1481,14 @@ display_properties_dialog (BonoboUIComponent *uic,
 	g_signal_connect (G_OBJECT (cd->props), "response",
 			  G_CALLBACK (properties_response_cb), cd);
 
+	/* Now set up the sensitivity based on gconf key writability */
+	setup_writability_sensitivity (cd, option_menu, label, KEY_HOUR_FORMAT);
+	setup_writability_sensitivity (cd, option_menu, label, KEY_UNIX_TIME);
+	setup_writability_sensitivity (cd, option_menu, label, KEY_INTERNET_TIME);
+	setup_writability_sensitivity (cd, showseconds, NULL, KEY_SHOW_SECONDS);
+	setup_writability_sensitivity (cd, showdate, NULL, KEY_SHOW_DATE);
+	setup_writability_sensitivity (cd, use_gmt_time, NULL, KEY_GMT_TIME);
+
 	/* sets up atk relation  */
 	list = g_slist_append (NULL, twelvehour);
 	list = g_slist_append (list, twentyfourhour);
@@ -1468,7 +1528,7 @@ display_help_dialog (BonoboUIComponent *uic,
 		dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_MODAL,
 						 GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_CLOSE,
+						 GTK_BUTTONS_OK,
 						  _("There was an error displaying help: %s"),
 						 error->message);
 
@@ -1489,7 +1549,6 @@ display_about_dialog (BonoboUIComponent *uic,
 		      ClockData         *cd,
 		      const gchar       *verbname)
 {
-	static GtkWidget *about = NULL;
 	GdkPixbuf *pixbuf = NULL;
 	gchar *file;
 	
@@ -1507,10 +1566,10 @@ display_about_dialog (BonoboUIComponent *uic,
 	/* Translator credits */
 	const char *translator_credits = _("translator_credits");
 
-	if (about) {
-		gtk_window_set_screen (GTK_WINDOW (about),
+	if (cd->about) {
+		gtk_window_set_screen (GTK_WINDOW (cd->about),
 				       gtk_widget_get_screen (cd->applet));
-		gtk_window_present (GTK_WINDOW (about));
+		gtk_window_present (GTK_WINDOW (cd->about));
 		return;
 	}
 
@@ -1524,7 +1583,7 @@ display_about_dialog (BonoboUIComponent *uic,
 	} else
 		g_warning (G_STRLOC ": gnome-clock.png cannot be found");
 
-	about = gnome_about_new (_("Clock"), VERSION,
+	cd->about = gnome_about_new (_("Clock"), VERSION,
 				 "Copyright \xc2\xa9 1998-2002 Free Software Foundation. Inc.",
 				 _("The Clock displays the current time and date"),
 				 authors,
@@ -1532,19 +1591,19 @@ display_about_dialog (BonoboUIComponent *uic,
 				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 	
-	gtk_window_set_wmclass (GTK_WINDOW (about), "clock", "Clock");
-	gtk_window_set_screen (GTK_WINDOW (about),
+	gtk_window_set_wmclass (GTK_WINDOW (cd->about), "clock", "Clock");
+	gtk_window_set_screen (GTK_WINDOW (cd->about),
 			       gtk_widget_get_screen (cd->applet));
 
 	if (pixbuf) {
-		gtk_window_set_icon (GTK_WINDOW (about), pixbuf);
+		gtk_window_set_icon (GTK_WINDOW (cd->about), pixbuf);
 		g_object_unref (pixbuf);
 	}
 	
-	g_signal_connect (G_OBJECT(about), "destroy",
-			  (GCallback)gtk_widget_destroyed, &about);
+	g_signal_connect (G_OBJECT(cd->about), "destroy",
+			  (GCallback)gtk_widget_destroyed, &cd->about);
 	
-	gtk_widget_show (about);
+	gtk_widget_show (cd->about);
 }
 
 static gboolean
