@@ -56,6 +56,8 @@ panel_applet_set_dnd_enabled (AppletInfo *info,
 		break;
 	case PANEL_OBJECT_BONOBO:
 		break;
+	case PANEL_OBJECT_LOGOUT:
+	case PANEL_OBJECT_LOCK:
 	case PANEL_OBJECT_ACTION:
 		panel_action_button_set_dnd_enabled (PANEL_ACTION_BUTTON (info->widget),
 						     dnd_enabled);
@@ -177,9 +179,6 @@ applet_callback_callback (GtkWidget      *widget,
 			launcher_properties (
 				menu->info->data, screen);
 
-		else if (!strcmp (menu->name, "help"))
-			panel_show_help (screen, "wgospanel.xml", "gospanel-16");
-
 		else if (!strcmp (menu->name, "help_on_app"))
 			launcher_show_help (menu->info->data, screen);
 		break;
@@ -463,6 +462,14 @@ panel_applet_create_menu (AppletInfo  *info,
 	if (!commie_mode) {
 		GtkWidget *image;
 		gboolean   locked;
+		gboolean   lockable;
+		gboolean   movable;
+		gboolean   removable;
+
+
+		lockable = panel_applet_lockable (info);
+		movable = panel_applet_can_freely_move (info);
+		removable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
 
 		locked = panel_widget_get_applet_locked (panel_widget, info->widget);
 
@@ -477,6 +484,7 @@ panel_applet_create_menu (AppletInfo  *info,
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (applet_remove_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, removable);
 
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
@@ -484,14 +492,15 @@ panel_applet_create_menu (AppletInfo  *info,
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (panel_applet_lock), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, lockable);
 		
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new ();
 		setup_menuitem (menuitem, GTK_ICON_SIZE_MENU, image, _("_Move"));
-		gtk_widget_set_sensitive (menuitem, !locked);
 		g_signal_connect (menuitem, "activate",
 				  G_CALLBACK (move_applet_callback), info);
 		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+		gtk_widget_set_sensitive (menuitem, !locked && movable);
 
 		g_assert (info->move_item == NULL);
 
@@ -874,7 +883,9 @@ panel_applet_save_position (AppletInfo *applet_info,
 	const char        *profile;
 	const char        *key;
 	const char        *toplevel_id;
+	char              *old_toplevel_id;
 	gboolean           right_stick;
+	gboolean           locked;
 	int                position;
 
 	g_return_if_fail (applet_info != NULL);
@@ -903,25 +914,33 @@ panel_applet_save_position (AppletInfo *applet_info,
 	
 	panel_widget = PANEL_WIDGET (applet_info->widget->parent);
 
-	key = panel_gconf_full_key (key_type, profile, id, "toplevel_id");
-	gconf_client_set_string (client, key, toplevel_id, NULL);
+	/* FIXME: Instead of getting keys, comparing and setting, there
+	   should be a dirty flag */
 
-	right_stick = panel_is_applet_right_stick (applet_info->widget);
+	key = panel_gconf_full_key (key_type, profile, id, "toplevel_id");
+	old_toplevel_id = gconf_client_get_string (client, key, NULL);
+	if (old_toplevel_id == NULL || strcmp (old_toplevel_id, toplevel_id) != 0)
+		gconf_client_set_string (client, key, toplevel_id, NULL);
+	g_free (old_toplevel_id);
+
+	right_stick = panel_is_applet_right_stick (applet_info->widget) ? 1 : 0;
 	key = panel_gconf_full_key (
 			key_type, profile, id, "panel_right_stick");
-	gconf_client_set_bool (client, key, right_stick, NULL);
+	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != right_stick)
+		gconf_client_set_bool (client, key, right_stick, NULL);
 
 	position = panel_applet_get_position (applet_info);
 	if (right_stick && !panel_widget->packed)
 		position = panel_widget->size - position;
 
 	key = panel_gconf_full_key (key_type, profile, id, "position");
-	gconf_client_set_int (client, key, position, NULL);
+	if (gconf_client_get_int (client, key, NULL) != position)
+		gconf_client_set_int (client, key, position, NULL);
 	
+	locked = panel_widget_get_applet_locked (panel_widget, applet_info->widget) ? 1 : 0;
 	key = panel_gconf_full_key (key_type, profile, id, "locked");
-	gconf_client_set_bool (client, key, 
-			       panel_widget_get_applet_locked (panel_widget, applet_info->widget),
-			       NULL);
+	if (gconf_client_get_bool (client, key, NULL) ? 1 : 0 != locked)
+		gconf_client_set_bool (client, key, locked, NULL);
 }
 
 const char *
@@ -1087,3 +1106,36 @@ panel_applet_get_position (AppletInfo *applet)
 
 	return applet_data->pos;
 }
+
+/* True if all the keys relevant to moving are writable
+   (position, toplevel_id, panel_right_stick) */
+gboolean
+panel_applet_can_freely_move (AppletInfo *applet)
+{
+	GConfClient *client = panel_gconf_get_client ();
+	const char *profile = panel_profile_get_name ();
+	PanelGConfKeyType key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	const char *key;
+       
+	key = panel_gconf_full_key (key_type, profile, applet->id, "position");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_gconf_full_key (key_type, profile, applet->id, "toplevel_id");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	key = panel_gconf_full_key (key_type, profile, applet->id, "panel_right_stick");
+	if ( ! gconf_client_key_is_writable (client, key, NULL))
+		return FALSE;
+	return TRUE;
+}
+
+/* True if the locked flag is writable */
+gboolean
+panel_applet_lockable (AppletInfo *applet)
+{
+	GConfClient *client = panel_gconf_get_client ();
+	const char *profile = panel_profile_get_name ();
+	PanelGConfKeyType key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+	const char *key = panel_gconf_full_key (key_type, profile, applet->id, "locked");
+	return gconf_client_key_is_writable (client, key, NULL);
+}
