@@ -28,6 +28,7 @@
 #include "panel-util.h"
 #include "xstuff.h"
 #include "panel-globals.h"
+#include "panel-stock-icons.h"
 
 #undef DRAWER_DEBUG
 
@@ -472,9 +473,6 @@ create_drawer_applet (PanelToplevel    *toplevel,
 		      const char       *pixmap,
 		      PanelOrientation  orientation)
 {
-        static GtkTargetEntry dnd_targets[] = {
-		{ "application/x-panel-applet-internal", 0, 0 }
-	};
 	Drawer *drawer;
 	
 	drawer = g_new0 (Drawer, 1);
@@ -486,32 +484,21 @@ create_drawer_applet (PanelToplevel    *toplevel,
 	else
 		drawer->tooltip = g_strdup (tooltip);
 
-	if (!pixmap || !pixmap [0])
-		drawer->pixmap = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
-							    "panel-drawer.png", TRUE, NULL);
-	else
+	if (!pixmap || !pixmap [0]) {
+		drawer->pixmap = NULL;
+		drawer->button = button_widget_new_from_stock (PANEL_STOCK_DRAWER,
+							       TRUE, orientation);
+	} else {
 		drawer->pixmap = g_strdup (pixmap);
+		drawer->button = button_widget_new (drawer->pixmap,
+						    TRUE, orientation);
+	}
 
-	drawer->button = button_widget_new (drawer->pixmap, -1,
-					    TRUE, orientation);
 	if (!drawer->button) {
 		free_drawer (drawer);
 		return NULL;
 	}
 
-	/*A hack since this function only pretends to work on window
-	  widgets (which we actually kind of are) this will select
-	  some (already selected) events on the panel instead of
-	  the button window (where they are also selected) but
-	  we don't mind*/ 
-	GTK_WIDGET_UNSET_FLAGS (drawer->button, GTK_NO_WINDOW);
- 
-	gtk_drag_source_set (drawer->button,
-			     GDK_BUTTON1_MASK,
-			     dnd_targets, 1,
-			     GDK_ACTION_MOVE);
-	GTK_WIDGET_SET_FLAGS (drawer->button, GTK_NO_WINDOW);
-
 	gtk_drag_dest_set (drawer->button, 0, NULL, 0, 0); 
 
 	g_signal_connect (drawer->button, "drag_data_get",
@@ -607,6 +594,7 @@ load_drawer_applet (char          *toplevel_id,
 		    const char    *pixmap,
 		    const char    *tooltip,
 		    PanelToplevel *parent_toplevel,
+		    gboolean       locked,
 		    int            pos,
 		    gboolean       exactpos,
 		    const char    *id)
@@ -614,6 +602,7 @@ load_drawer_applet (char          *toplevel_id,
 	PanelOrientation  orientation;
 	PanelToplevel    *toplevel = NULL;
 	Drawer           *drawer = NULL;
+	PanelWidget      *panel_widget;
 
 	orientation = panel_toplevel_get_orientation (parent_toplevel);
 
@@ -629,10 +618,13 @@ load_drawer_applet (char          *toplevel_id,
 	if (!drawer)
 		return;
 
+	panel_widget = panel_toplevel_get_panel_widget (parent_toplevel);
+
 	drawer->info = panel_applet_register (drawer->button, drawer,
 					      (GDestroyNotify) free_drawer,
-					      panel_toplevel_get_panel_widget (parent_toplevel),
-					      pos, exactpos, PANEL_OBJECT_DRAWER, id);
+					      panel_widget,
+					      locked, pos, exactpos,
+					      PANEL_OBJECT_DRAWER, id);
 
 	if (!drawer->info) {
 		gtk_widget_destroy (GTK_WIDGET (toplevel));
@@ -643,6 +635,7 @@ load_drawer_applet (char          *toplevel_id,
 				G_CALLBACK (drawer_button_size_allocated), drawer);
 
 	panel_widget_add_forbidden (panel_toplevel_get_panel_widget (drawer->toplevel));
+	panel_widget_set_applet_expandable (panel_widget, GTK_WIDGET (drawer->button), FALSE, TRUE);
 
 	gtk_tooltips_set_tip (panel_tooltips, drawer->button, drawer->tooltip, NULL);
 	gtk_window_present (GTK_WINDOW (drawer->toplevel));
@@ -699,6 +692,7 @@ panel_drawer_delete (Drawer *drawer)
 
 void
 drawer_load_from_gconf (PanelWidget *panel_widget,
+			gboolean     locked,
 			gint         position,
 			const char  *id)
 {
@@ -734,6 +728,7 @@ drawer_load_from_gconf (PanelWidget *panel_widget,
 			    pixmap,
 			    tooltip,
 			    panel_widget->toplevel,
+			    locked,
 			    position,
 			    TRUE,
 			    id);
@@ -742,3 +737,24 @@ drawer_load_from_gconf (PanelWidget *panel_widget,
 	g_free (pixmap);
 	g_free (tooltip);
 }
+
+void
+panel_drawer_set_dnd_enabled (Drawer   *drawer,
+			      gboolean  dnd_enabled)
+{
+	if (dnd_enabled) {
+		static GtkTargetEntry dnd_targets[] = {
+			{ "application/x-panel-applet-internal", 0, 0 }
+		};
+
+		GTK_WIDGET_UNSET_FLAGS (drawer->button, GTK_NO_WINDOW);
+		gtk_drag_source_set (drawer->button,
+				     GDK_BUTTON1_MASK,
+				     dnd_targets, 1,
+				     GDK_ACTION_MOVE);
+		gtk_drag_source_set_icon_stock (drawer->button, PANEL_STOCK_DRAWER);
+		GTK_WIDGET_SET_FLAGS (drawer->button, GTK_NO_WINDOW);
+
+	} else
+		gtk_drag_source_unset (drawer->button);
+}
