@@ -24,11 +24,6 @@ void       menu_item_menu_position   (GtkMenu  *menu,
 				      gboolean *push_in,
 				      gpointer  data);
 
-char *     get_real_menu_path      (const char *arguments,
-                                    gboolean main_menu);
-char *     get_pixmap              (const char *menudir,
-                                    gboolean main_menu);
-
 GtkWidget *add_menu_separator     (GtkWidget *menu);
 
 GtkWidget *stock_menu_item_new (const char *text,
