@@ -46,26 +46,30 @@ char          *panel_profile_find_new_id        (PanelGConfKeyType  type);
 
 gboolean    panel_profile_get_show_program_list   (void);
 void        panel_profile_set_show_program_list   (gboolean show_program_list);
+gboolean    panel_profile_is_writable_show_program_list (void);
 gboolean    panel_profile_get_enable_program_list (void);
 
 
-void        panel_profile_add_to_list            (PanelGConfKeyType  type,
-						  char              *id);
-void        panel_profile_remove_from_list       (PanelGConfKeyType  type,
-						  const char        *id);
-void        panel_profile_create_toplevel        (void);
-void        panel_profile_load_toplevel          (GConfClient       *client,
-						  const char        *profile_dir,
-						  PanelGConfKeyType  type,
-						  char              *toplevel_id);
-void        panel_profile_delete_toplevel        (PanelToplevel     *toplevel);
-char       *panel_profile_prepare_object         (PanelObjectType    object_type,
-						  PanelToplevel     *toplevel,
-						  int                position);
-char       *panel_profile_prepare_object_with_id (PanelObjectType  object_type,
-						  const char      *toplevel_id,
-						  int              position);
-void        panel_profile_delete_object          (AppletInfo        *applet_info);
+void           panel_profile_add_to_list            (PanelGConfKeyType  type,
+						     char              *id);
+void           panel_profile_remove_from_list       (PanelGConfKeyType  type,
+						     const char        *id);
+gboolean       panel_profile_list_is_writable       (PanelGConfKeyType  type);
+void           panel_profile_create_toplevel        (void);
+PanelToplevel *panel_profile_load_toplevel          (GConfClient       *client,
+						     const char        *profile_dir,
+						     PanelGConfKeyType  type,
+						     char              *toplevel_id);
+void           panel_profile_delete_toplevel        (PanelToplevel     *toplevel);
+char          *panel_profile_prepare_object         (PanelObjectType    object_type,
+						     PanelToplevel     *toplevel,
+						     int                position,
+						     gboolean           right_stick);
+char          *panel_profile_prepare_object_with_id (PanelObjectType    object_type,
+						     const char        *toplevel_id,
+						     int                position,
+						     gboolean           right_stick);
+void           panel_profile_delete_object          (AppletInfo        *applet_info);
 
 
 gboolean panel_profile_is_toplevel_list_writeable (void);
@@ -74,53 +78,80 @@ gboolean panel_profile_is_toplevel_list_writeable (void);
 void        panel_profile_set_toplevel_name           (PanelToplevel *toplevel,
 						       const char    *name);
 char       *panel_profile_get_toplevel_name           (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_name   (PanelToplevel *toplevel);
+
 void        panel_profile_set_toplevel_orientation    (PanelToplevel *toplevel,
 						       PanelOrientation orientation);
 PanelOrientation
             panel_profile_get_toplevel_orientation    (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_orientation (PanelToplevel *toplevel);
+
 void        panel_profile_set_toplevel_size           (PanelToplevel *toplevel,
 						       int            size);
 int         panel_profile_get_toplevel_size           (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_size   (PanelToplevel *toplevel);
+
 void        panel_profile_set_toplevel_expand         (PanelToplevel *toplevel,
 						       gboolean       expand);
 gboolean    panel_profile_get_toplevel_expand         (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_expand (PanelToplevel *toplevel);
+
 void        panel_profile_set_toplevel_auto_hide      (PanelToplevel *toplevel,
 						       gboolean       auto_hide);
 gboolean    panel_profile_get_toplevel_auto_hide      (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_auto_hide (PanelToplevel *toplevel);
+
 void        panel_profile_set_toplevel_enable_buttons (PanelToplevel *toplevel,
 						       gboolean       enable_buttons);
 gboolean    panel_profile_get_toplevel_enable_buttons (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_enable_buttons (PanelToplevel *toplevel);
+
 void        panel_profile_set_toplevel_enable_arrows  (PanelToplevel *toplevel,
 						       gboolean       enable_arrows);
 gboolean    panel_profile_get_toplevel_enable_arrows  (PanelToplevel *toplevel);
+gboolean    panel_profile_is_writable_toplevel_enable_arrows (PanelToplevel *toplevel);
 
 void        panel_profile_set_background_type         (PanelToplevel       *toplevel,
 						       PanelBackgroundType  background_type);
 PanelBackgroundType
 	    panel_profile_get_background_type         (PanelToplevel       *toplevel);
+gboolean    panel_profile_is_writable_background_type (PanelToplevel       *toplevel);
+
 void        panel_profile_set_background_color        (PanelToplevel       *toplevel,
 						       PanelColor          *color);
 void        panel_profile_get_background_color        (PanelToplevel       *toplevel,
 						       PanelColor          *color);
+gboolean    panel_profile_is_writable_background_color (PanelToplevel       *toplevel);
+
 void        panel_profile_set_background_pango_color  (PanelToplevel       *toplevel,
 						       PangoColor          *pango_color);
 void        panel_profile_get_background_pango_color  (PanelToplevel       *toplevel,
 						       PangoColor          *pango_color);
+
 void        panel_profile_set_background_opacity      (PanelToplevel       *toplevel,
 						       guint16              opacity);
 guint16     panel_profile_get_background_opacity      (PanelToplevel       *toplevel);
+gboolean    panel_profile_is_writable_background_opacity (PanelToplevel       *toplevel);
+
 void        panel_profile_set_background_image        (PanelToplevel       *toplevel,
 						       const char          *image);
 char       *panel_profile_get_background_image        (PanelToplevel       *toplevel);
+gboolean    panel_profile_is_writable_background_image (PanelToplevel       *toplevel);
+
 void        panel_profile_set_background_fit          (PanelToplevel       *toplevel,
 						       gboolean             fit);
 gboolean    panel_profile_get_background_fit          (PanelToplevel       *toplevel);
+gboolean    panel_profile_is_writable_background_fit  (PanelToplevel *toplevel);
+
 void        panel_profile_set_background_stretch      (PanelToplevel       *toplevel,
 						       gboolean             stretch);
 gboolean    panel_profile_get_background_stretch      (PanelToplevel       *toplevel);
+gboolean    panel_profile_is_writable_background_stretch (PanelToplevel *toplevel);
+
 void        panel_profile_set_background_rotate       (PanelToplevel       *toplevel,
 						       gboolean             rotate);
 gboolean    panel_profile_get_background_rotate       (PanelToplevel       *toplevel);
+gboolean    panel_profile_is_writable_background_rotate (PanelToplevel *toplevel);
 
 guint       panel_profile_toplevel_notify_add         (PanelToplevel         *toplevel,
 						       const char            *key,
@@ -138,6 +169,9 @@ gboolean    panel_profile_map_background_type_string  (const char            *st
 gboolean    panel_profile_map_object_type_string      (const char            *str,
 						       PanelObjectType       *object_type);
 
+/* all keys relevant to moving are writable */
+gboolean    panel_profile_can_be_moved_freely         (PanelToplevel *toplevel);
+
 G_END_DECLS
 
 #endif /* __PANEL_PROFILE_H__ */
